@@ -369,6 +369,8 @@ void pad_archive (off_t size_left);
 void dump_file (char *, int, dev_t);
 union block *start_header (struct tar_stat_info *st);
 void finish_header (struct tar_stat_info *, union block *, off_t);
+void simple_finish_header (union block *header);
+union block *start_private_header (const char *name, size_t size);
 void write_eot (void);
 void check_links (void);
 
@@ -501,7 +503,7 @@ enum remove_option
   WANT_DIRECTORY_REMOVE_OPTION
 };
 int remove_any_file (const char *, enum remove_option);
-int maybe_backup_file (const char *, int);
+bool maybe_backup_file (const char *, int);
 void undo_last_backup (void);
 
 int deref_stat (bool, char const *, struct stat *);
@@ -622,9 +624,12 @@ void xheader_store (char const *, struct tar_stat_info const *, void *);
 void xheader_read (union block *, size_t);
 void xheader_finish (struct xheader *);
 void xheader_destroy (struct xheader *);
+char *xheader_xhdr_name (struct tar_stat_info *st);
+char *xheader_ghdr_name (void);
 
 /* Module system.c */
 
+void sys_stat_nanoseconds (struct tar_stat_info *stat);
 void sys_detect_dev_null_output (void);
 void sys_save_archive_dev_ino (void);
 void sys_drain_input_pipe (void);
