@@ -130,10 +130,19 @@ read_and (void (*do_something) (void))
 	    }
 
 	  set_next_block_after (current_header);
+
+	  if (!ignore_zeros_option)
+	    {
+	      char buf[UINTMAX_STRSIZE_BOUND];
+
+	      status = read_header (false);
+	      if (status == HEADER_ZERO_BLOCK)
+		break;
+	      WARN ((0, 0, _("A lone zero block at %s"),
+		     STRINGIFY_BIGINT (current_block_ordinal (), buf)));
+	    }
 	  status = prev_status;
-	  if (ignore_zeros_option)
-	    continue;
-	  break;
+	  continue;
 
 	case HEADER_END_OF_FILE:
 	  if (block_number_option)
@@ -386,9 +395,13 @@ read_header (bool raw_extended_headers)
 
 	      *bp = '\0';
 	    }
-	  else if (header->header.typeflag == XHDTYPE
-		   || header->header.typeflag == XGLTYPE)
+	  else if (header->header.typeflag == XHDTYPE)
 	    xheader_read (header, OFF_FROM_HEADER (header->header.size));
+	  else if (header->header.typeflag == XGLTYPE)
+	    {
+	      xheader_read (header, OFF_FROM_HEADER (header->header.size));
+	      xheader_decode_global ();
+	    }
       
 	  /* Loop!  */
 
@@ -559,8 +572,7 @@ decode_header (union block *header, struct tar_stat_info *stat_info,
     }
 
   current_stat_info.archive_file_size = current_stat_info.stat.st_size;
-  if (extended_header.size)
-    xheader_decode (stat_info);
+  xheader_decode (stat_info);
 }
 
 /* Convert buffer at WHERE0 of size DIGS from external format to
