@@ -3,6 +3,7 @@
 
 . ./preset
 . $srcdir/before
+prereq gnu oldgnu
 
 set -e
 mkdir structure
@@ -22,12 +23,12 @@ ls -l structure/file >/dev/null
 # backing it up.
 sleep 1
 
-tar cf archive --format=gnu --listed=list structure
-tar cfv archive --format=gnu --listed=list structure
+tar cf archive --listed=list structure
+tar cfv archive --listed=list structure
 echo -----
 sleep 1
 echo y >structure/file
-tar cfv archive --format=gnu --listed=list structure
+tar cfv archive --listed=list structure
 
 out="\
 structure/
