@@ -3,13 +3,14 @@
 
 . ./preset
 . $srcdir/before
+prereq gnu oldgnu
 
 # Fixme: should be configurable
 #  TRUSS=truss -o /tmp/tr
 #  TRUSS=strace
 set -e
 
-dd if=/dev/zero bs=1024 count=7 >file1
+dd if=/dev/zero bs=1024 count=7 2>/dev/null >file1
 
 for block in " 1" " 2" " 3" " 4" " 5" " 6" " 7" " 8" \
               " 9" "10" "11" "12" "13" "14" "15" "16" ; do \
@@ -19,13 +20,13 @@ for block in " 1" " 2" " 3" " 4" " 5" " 6" " 7" " 8" \
   done
 done >file2
 
-tar -c --format=gnu --multi-volume --tape-length=10 \
+tar -c --multi-volume --tape-length=10 \
   --listed-incremental=t.snar \
   -f t1-pipe.tar -f t2-pipe.tar ./file1 ./file2
 
 mkdir extract-dir-pipe
-dd bs=4096 count=10 if=t2-pipe.tar |
-PATH=$PATH ${TRUSS} tar --format=gnu -f t1-pipe.tar -f - \
+dd bs=4096 count=10 if=t2-pipe.tar 2>/dev/null |
+PATH=$PATH ${TRUSS} tar -f t1-pipe.tar -f - \
       -C extract-dir-pipe -x --multi-volume \
       --tape-length=10 --read-full-records
 
@@ -34,12 +35,5 @@ cmp file2 extract-dir-pipe/file2
 
 out="\
 "
-err="\
-7+0 records in
-7+0 records out
-tar: Removing leading \`./' from member names
-2+1 records in
-2+1 records out
-"
 
 . $srcdir/after
